/*
Copyright (c) 2003-2004, Gregory Kuhlmann, Peter Stone, Justin Lallinger,
University of Texas at Austin
All right reserved

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the University of Texas nor the names of its
contributors may be used to endorse or promote products derived from this
software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <iostream>
#include "ClangTypes.h"
#include "ClangCondition.h"
#include "ClangRuleBase.h"
#include "ClangRule.h"

// PPosCondition stuff

PPosCondition::PPosCondition( CLTeam t, UnumSet * s, int minplrs, int maxplrs,
			      Region * r )
{
  this->t = t;
  this->s = s;
  this->minplrs = minplrs;
  this->maxplrs = maxplrs;
  this->reg = r;
}

PPosCondition::PPosCondition()
{
  s = NULL;
  reg = NULL;
}

PPosCondition::~PPosCondition()
{
  delete s;
  delete reg;
}


bool PPosCondition::Evaluate( Rule * r )
{
  s->ClearInstantiation();

  for ( int i = 1; i <= 11; i++ ) {
    if ( reg->PointInRegion( CLGetPlayerX( t, i ), CLGetPlayerY( t, i ) ) )
      s->Instantiate( i );
  }

  if ( r ) {
    r->InstantiateVariable( s );
  }

  int p = 0;
  for ( int i = 1; i <= 11; i++ ) {
    if (!CLGetPlayerKnown(t,i))
      continue;
    if ( s->IsSet( i, false )
	 && reg->PointInRegion( CLGetPlayerX( t, i ), CLGetPlayerY( t, i ) ) )
      p++;
  }

  if ( p >= minplrs && p <= maxplrs ) {
    return true;
  }
  return false;
}

void PPosCondition::Output( ostream & os, bool dv )
{
  os << "(ppos ";
  if ( t == our )
    os << "our ";
  else
    os << "opp ";

  s->Output( os, dv );
  os << ' ' << minplrs << ' ' << maxplrs << ' ';
  reg->Output( os );
  os << ')';
}


// BPosCondition stuff
BPosCondition::BPosCondition()
{
  reg = NULL;
}

BPosCondition::BPosCondition( Region * reg )
{
  this->reg = reg;
}

bool BPosCondition::Evaluate( Rule * r )
{
  return reg->PointInRegion( CLGetBallX(), CLGetBallY() );
}

void BPosCondition::Output( ostream & os, bool dv )
{
  os << "(bpos ";
  reg->Output( os );
  os << ')';
}

// BOwnerCondition stuff
BOwnerCondition::BOwnerCondition()
{
  s = NULL;
}

BOwnerCondition::BOwnerCondition( CLTeam t, UnumSet * s )
{
  this->t = t;
  this->s = s;
}

bool BOwnerCondition::Evaluate( Rule * r )
{
  s->ClearInstantiation();
  if ( CLGetBallOwnerTeam() == t )
    s->Instantiate( CLGetBallOwnerUnum() );
  if ( r )
    r->InstantiateVariable( s );
  return ( CLGetBallOwnerTeam() == t )
    && ( s->IsSet( CLGetBallOwnerUnum(), true )
	 || s->IsSet( CLGetBallOwnerUnum(), false ) );
}

void BOwnerCondition::Output( ostream & os, bool dv )
{
  os << "(bowner ";
  if ( t == opp )
    os << "opp ";
  else
    os << "our ";
  s->Output( os, dv );
  os << ')';
}

// UnumCondition stuff

UnumCondition::UnumCondition()
{
  v = NULL;
  u = NULL;
  n = NULL;
}

UnumCondition::~UnumCondition()
{
  delete v;
  delete u;
  delete n;
}


UnumCondition::UnumCondition( CLangVar * v, UnumSet * u )
{
  this->v = v;
  this->u = u;
  n = new UnumSet;
}

bool UnumCondition::Evaluate( Rule * r )
{
  if ( !r )
    return false;
  UnumSet *iv = r->GetInstantiatedVariable( v );
  if ( iv == NULL ) {
    n->SetCLangVar( v );
    n->Clear();
    //int toset = 0;
    if ( u->ContainsVariable() ) {
      UnumSet *ui = r->GetInstantiatedVariable( u->GetCLangVar() );
      u->ClearInstantiation();
      for ( int i = 1; i <= 11; i++ ) {
	if ( ui->IsSet( i ) ) {
	  u->Instantiate( i );
	}
      }

    }
    for ( int i = 1; i <= 11; i++ ) {
      if ( u->IsSet( i ) ) {

	n->Instantiate( i );
      }
    }



    r->InstantiateVariable( n );
    return true;
  }
  else {
    n->ClearInstantiation();
    n->SetCLangVar( v );
    for ( int i = 1; i <= 11; i++ ) {
      if ( iv->IsSet( i, true ) )
	n->Instantiate( i );
    }
    for ( int i = 1; i <= 11; i++ ) {
      if ( u->IsSet( i, false ) != iv->IsSet( i, true ) )
	return false;
    }
    return true;
  }
}

void UnumCondition::Output( ostream & os, bool dv )
{
  os << "(unum ";
  v->Output( os );

  os << ' ';
  u->Output( os, dv );
  os << ')';
}


PlayMCondition::PlayMCondition( CLPlayMode pm )
{
  this->pm = pm;
}

bool PlayMCondition::Evaluate( Rule * r )
{
  return CLGetPlayMode() == pm;
}

void PlayMCondition::Output( ostream & os, bool dv )
{
  os << "(playm ";
  switch ( pm ) {
  case bko:
    os << "bko";
    break;
  case time_over:
    os << "time_over";
    break;
  case play_on:
    os << "play_on";
    break;
  case ko_our:
    os << "ko_our";
    break;
  case ko_opp:
    os << "ko_opp";
    break;
  case ki_our:
    os << "ki_our";
    break;
  case ki_opp:
    os << "ki_opp";
    break;
  case fk_our:
    os << "fk_our";
    break;
  case fk_opp:
    os << "fk_opp";
    break;
  case ck_our:
    os << "ck_our";
    break;
  case ck_opp:
    os << "ck_opp";
    break;
  case gk_our:
    os << "gk_our";
    break;
  case gk_opp:
    os << "gk_opp";
    break;
  case gc_our:
    os << "gc_our";
    break;
  case gc_opp:
    os << "gc_opp";
    break;
  case ag_our:
    os << "ag_our";
    break;
  case ag_opp:
    os << "ag_opp";
    break;
  }



  os << ')';

}


// CondComp stuff here

CondComp::CondComp( CondCompOperandType ot1, double const1, CondCompOp op,
		    CondCompOperandType ot2, double const2 )
{
  this->ot1 = ot1;
  this->const1 = const1;
  this->op = op;
  this->ot2 = ot2;
  this->const2 = const2;
}

bool CondComp::Evaluate( Rule * r )
{
  double v1 = 0, v2 = 0;


  switch ( ot1 ) {
  case constant:
    v1 = const1;
    break;
  case svtime:
    v1 = CLGetTime();
    break;
  case ourgoals:
    v1 = CLGetScore( our );
    break;
  case oppgoals:
    v1 = CLGetScore( opp );
    break;
  case goaldiff:
    v1 = CLGetScore( our ) - CLGetScore( opp );
    break;
  }

  switch ( ot2 ) {
  case constant:
    v2 = const2;
    break;
  case svtime:
    v2 = CLGetTime();
    break;
  case ourgoals:
    v2 = CLGetScore( our );
    break;
  case oppgoals:
    v2 = CLGetScore( opp );
    break;
  case goaldiff:
    v2 = CLGetScore( our ) - CLGetScore( opp );
    break;
  }


  switch ( op ) {
  case lt:
    return v1 < v2;
    break;
  case gt:
    return v1 > v2;
    break;
  case eq:
    return v1 == v2;
    break;
  case lte:
    return v1 <= v2;
    break;
  case gte:
    return v1 >= v2;
    break;
  case neq:
    return v1 != v2;
    break;
  }
  return false;
}

void CondComp::Output( ostream & os, bool dv )
{
  os << '(';
  switch ( ot1 ) {
  case constant:
    os << const1;
    break;
  case svtime:
    os << "time";
    break;
  case ourgoals:
    os << "our_goals";
    break;
  case oppgoals:
    os << "opp_goals";
    break;
  case goaldiff:
    os << "goal_diff";
    break;
  }

  switch ( op ) {
  case lt:
    os << " < ";
    break;
  case gt:
    os << " > ";
    break;
  case eq:
    os << " == ";
    break;
  case lte:
    os << " <= ";
    break;
  case gte:
    os << " >= ";
    break;
  case neq:
    os << " |= ";
    break;
  }


  switch ( ot2 ) {
  case constant:
    os << const2;
    break;
  case svtime:
    os << "time";
    break;
  case ourgoals:
    os << "our_goals";
    break;
  case oppgoals:
    os << "opp_goals";
    break;
  case goaldiff:
    os << "goal_diff";
    break;
  }

  os << ')';
}

// AndCondition stuff

AndCondition::~AndCondition()
{
  conditionList *c = l, *t;
  while ( c ) {
    t = c;
    c = c->next;
    delete t->cond;
  }
}

bool AndCondition::Evaluate( Rule * r )
{
  conditionList *c = l;
  while ( c ) {
    if ( c->cond->Evaluate( r ) == false )
      return false;
    c = c->next;
  }
  return true;
}

void AndCondition::Output( ostream & os, bool dv )
{
  os << "(and ";
  conditionList *c = l;
  while ( c ) {
    c->cond->Output( os, dv );
    if ( c->next )
      os << ' ';
    c = c->next;
  }
  os << ')';

}

// OrCondition stuff

OrCondition::~OrCondition()
{
  conditionList *c = l, *t;
  while ( c ) {
    t = c;
    c = c->next;
    delete t->cond;
  }
}

bool OrCondition::Evaluate( Rule * r )
{
  conditionList *c = l;
  while ( c ) {
    if ( c->cond->Evaluate( r ) == true )
      return true;
    c = c->next;
  }
  return false;
}

void OrCondition::Output( ostream & os, bool dv )
{
  os << "(or ";
  conditionList *c = l;
  while ( c ) {
    c->cond->Output( os, dv );
    if ( c->next )
      os << ' ';
    c = c->next;
  }
  os << ')';

}

// NotCondition Stuff
void NotCondition::Output( ostream & os, bool dv )
{
  os << "(not ";
  c->Output( os );
  os << ')';
}

// VarCondition Stuff

void VarCondition::Output( ostream & os, bool dv )
{
  v->Output( os );
}

bool VarCondition::Evaluate( Rule * r )
{
  return rulebase->ResolveCondition( v )->Evaluate( r );
}
