/*
Copyright (c) 2003-2004, Gregory Kuhlmann, Peter Stone, Justin Lallinger,
University of Texas at Austin
All right reserved

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the University of Texas nor the names of its
contributors may be used to endorse or promote products derived from this
software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef _CLANG_EVAL_
#define _CLANG_EVAL_

#include "ClangDirective.h"
#include "StockAction.h"
#include "SoccerTypes.h"
#include "WorldModel.h"
#include "PlayerSettings.h"
#include "ClangRBInterface.h"

enum CoachAdvice
{
  ADVISED_DONT = 0,
  NO_ADVICE,
  ADVISED_DO
} ;

struct CLEvalList
{
  Action *act;
  double eval;
  CLEvalList *next;
} ;

class CLEval
{
public:
  CLEval( WorldModel *wm, ServerSettings *ss, PlayerSettings *ps, char *inFile = NULL );
  ~CLEval();

  void SetWorldModel( WorldModel * wm )
  {
    this->wm = wm;
  }
  Action *GetBestAction();
  VecPosition GetHomePos()
  {
    return homepos;
  }
  VecPosition GetPosPos()
  {
    return pospos;
  }
  VecPosition GetShootPoint()
  {
    return shotpt;
  }
  VecPosition GetPassPoint( int i )
  {
    return passpt[i - 1];
  }
  double GetPassSpeed( int i )
  {
    return passspd[i - 1];
  }
  int GetPassTime( int i )
  {
    return passtime[i];
  }

  double GetPassTargConf( int i )
  {
    return passtargconf[i-1];
  }
  void Update();
  double Evaluate( Action * act );
  void LoadOptions( char *fname );
  void GetMarkInfo( VecPosition * p, ObjectT * m );

  // Constants to control the players' behavior

  /// The amount that a coach "do" advice increases 
  /// the eval of an action       
  double doweight;

  /// The amount that a coach "dont" advice decreases
  /// the eval of an action
  double dontweight;

  /// The constant that's returned for a "go home" stock
  /// action, or a go pos stock action (if there's a pos available)
  double gohomeconst;

  /// The amount the intercept action gets for you being
  /// fastest to the ball
  double interceptfastest;

  /// The amount the intercept action gets for you being
  /// closest to the ball (but not the fastest)
  double interceptclosest;

  /// The amount the intercept action gets otherwise
  double interceptnotfastest;

  /// The amount that's returned for a "hold" stock action
  double holdconst;

  /// Consider passes within a range of +/- this value (in degrees) of each player
  double passanglerange;

  /// Consider passes with speed greater than or equal to this value
  double passspeedmin;

  /// Consider passes with speed less than or equal to this value
  double passspeedmax;

  /// Step value for angle portion of pass considerations
  double passanglestep;

  /// Step value for speed portion of pass considerations
  double passspeedstep;

  /// The "delta" component of the lessthandelta for calculating pass probs
  double passdelta;

  double cleardist;
  double shootdist;

  /// Pass value
  double passvalue;

  /// Forward pass bias
  double forwardpassbias;

  /// The step value for shot point considerations
  double shotstep;
  /// The value of a shot
  double shotvalue;
  /// The delta value to the lessthandelta function
  double shotdelta;

  /// The value of a clear
  double clearvalue;

  /// The value of a dribble
  double dribblevalue;

  double minpassconf;

  /// If there is < fastdribblethr opponents in a circle of dribblerad from player, DRIBBLE_FAST
  double fastdribblethr;
  /// else if there is < slowdribblethr opponents in a circle of dribblerad from player, DRIBBLE_SLOW, else DRIBBLE_WITH_BALL
  double slowdribblethr;
  /// Controls opponent search radius
  double dribblerad;

  /// The lessthandelta value for dribbles
  double dribbledelta;

  /// Points for clear left, right, and center stock actions
  double stockclrlx, stockclrly, stockclrrx, stockclrry, stockclrcx,
    stockclrcy;
  /// Points for dribble left, right, and center stock actions
  double stockdriblx, stockdribly, stockdribrx, stockdribry, stockdribcx,
    stockdribcy;

  double shotmin, shotmax;

  double markvalue;

  double ballattrx, ballattry, ballattrdist;
  double oppattrx, oppattry, oppattrdist;

  double markchargedist;

  VecPosition GetPassReceivePoint();
  void SetPassReceivePoint( VecPosition pt );

private:
  void Cleanup();
  void UpdatePassAdvice( Action * act, bool doordont );
  void UpdateMarkAdvice( Directive * act );
  void UpdateHomePosition( Region * r );
  void UpdatePosPosition( Region * r );

  WorldModel *wm;
  ServerSettings *ss;
  PlayerSettings *ps;
  CoachAdvice passadvised[11];
  CLEvalList *evals;
  VecPosition homepos;
  VecPosition pospos;
  VecPosition shotpt;
  bool usepos;
  Action **stockactions;
  int numStockActions;

  double passtargconf[11];

  double EvalIntercept();
  double EvalPass( int passto );

  double EvalShoot();
  double EvalDribble( Region * r );
  double EvalClear( Region * r );

  void ChooseShootPt();
  void ChoosePasses();
  void ChooseMarks();
  double InterceptionProb( VecPosition dest );
  double shoteval;
  bool DribbleUnadvised( double tx, double ty );
  bool ClearUnadvised( double tx, double ty );

  VecPosition passpt[11];
  double passspd[11];
  double passeval[11];
  int passtime[11];
  CoachAdvice interceptadvised;
  CoachAdvice shootadvised;
  CoachAdvice holdadvised;

  bool markmatrix[11][11];
  bool marklmatrix[11][11];

  VecPosition markpos;
  ObjectT mark;

  bool passtoreceive;
  VecPosition passreceivept;
  double markeval;
} ;

// Stone/McAllester soft boolean functions
inline double ifstar( double cond, double ifp, double elsep )
{
  return cond * ifp + ( 1 - cond ) * elsep;
}

inline double lessthandelta( double x, double y, double delta )
{
  return 1 / ( 1 + exp( -( y - x ) / delta ) );
}

inline double equaldelta( double x, double y, double delta )
{
  return exp( -( ( x - y ) * ( x - y ) ) / ( delta * delta ) );
}

#endif
