/*
Copyright (c) 2003-2004, Gregory Kuhlmann, Peter Stone, Justin Lallinger,
University of Texas at Austin
All right reserved

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the University of Texas nor the names of its
contributors may be used to endorse or promote products derived from this
software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <math.h>
#include "SoccerTypes.h"
#include "Geometry.h"
#include "ClangRegion.h"
#include "ClangRuleBase.h"
#include "ClangRule.h"

// Declare these functions, because they have to be declared somewhere
bool (*CLGetPlayerKnown)(CLTeam t, int p);
double ( *CLGetPlayerX ) ( CLTeam t, int p );
double ( *CLGetPlayerY ) ( CLTeam t, int p );
int ( *CLGetBallOwnerUnum ) ();
CLTeam( *CLGetBallOwnerTeam ) ();
double ( *CLGetBallX ) ();
double ( *CLGetBallY ) ();
int ( *CLGetScore ) ( CLTeam t );
int ( *CLGetTime ) ();
CLPlayMode( *CLGetPlayMode ) ();
bool( *CLReverseCoords ) ();

// Utility function to calculate the intersection of a line from a point
// to a circle's center and the circle itself

void CircleIntersect( double cx, double cy, double rad, double tx, double ty,
		      double *rx1, double *ry1, double *rx2, double *ry2 );

// PointRegion stuff

PointRegion::PointRegion()
{
  x = y = 0;
  usePlr = false;
}

/*!	This constructor creates a PointRegion representing the CLang
	construct (pt x y)
	\param x The x coordinate of the point
	\param y The y coordinate of the point

*/
PointRegion::PointRegion( double x, double y )
{
  if ( CLReverseCoords() ) {
    this->x = -x;
    this->y = -y;
  }
  else {
    this->x = x;
    this->y = y;
  }
  usePlr = false;
}

/*! 	This constructor creates a PointRegion representing one of 
	the CLang relative point constructs. If plr = 0, it creates a
	PointRegion representing (pt ball), otherwise it creates a
	PointRegion representing (pt t plr)
	
	\param t Either opp or our, the team of the player whose coordinates the point represents
	\param plr A unum from 1-11, or 0 to represent the ball
*/
PointRegion::PointRegion( CLTeam t, int plr )
{
  this->t = t;
  this->plr = plr;
  usePlr = true;
}

/*! 	Gets the X coordinate of the point
	\retval the x coordinate of the point
*/
double PointRegion::GetX()
{
  if ( usePlr && plr == 0 ) {

    return CLGetBallX();
  }
  else if ( usePlr ) {
    return CLGetPlayerX( t, plr );
  }
  else {
    return x;
  }

}

/*! 	Gets the Y coordinate of the point
	\retval the y coordinate of the point
*/
double PointRegion::GetY()
{
  if ( usePlr && plr == 0 ) {
    return CLGetBallY();
  }
  else if ( usePlr ) {
    return CLGetPlayerY( t, plr );
  }
  else {
    return y;
  }
}

/*! 	This tests to see whether or not the point represented by 
	(x,y) is within the region. For a PointRegion, this would 
	mean that (x,y) is the same point
	\param x The x coordinate of the point to test
	\param y The y coordinate of the point to test
	\retval true if (x,y) is within the region, false otherwise
*/
bool PointRegion::PointInRegion( double x, double y )
{
  return ( GetX() == x && GetY() == y );
}

/*! 	This function outputs the string in its CLang form
	\param os The stream to output to

*/
void PointRegion::Output( ostream & os, bool dv )
{
  if ( usePlr && plr == 0 ) {
    os << "(pt ball)";

  }
  else if ( usePlr ) {
    os << "(pt ";
    if ( t == our )
      os << "our ";
    else
      os << "opp ";
    os << plr << ")";

  }
  else {
    os << "(pt " << x << " " << y << ")";

  }

}

/*!	Finds the closest point to (x,y) that is inside of the region
	and stores it in (*rx, *ry). For a PointRegion, this is a simple 
	task, since there is only one point in the region

*/
void PointRegion::GetClosestPointInRegion( double x, double y, double *rx,
					   double *ry )
{
  *rx = GetX();
  *ry = GetY();
}

// PointArithRegion stuff

PointArithRegion::PointArithRegion()
{
  pt1 = pt2 = NULL;
  op = PTADD;
}

PointArithRegion::~PointArithRegion()
{
  delete pt1;
  delete pt2;
}

PointArithRegion::PointArithRegion( PointRegion * pt1, PointRegion * pt2,
				    PtArithOp op )
{
  this->pt1 = pt1;
  this->pt2 = pt2;
  this->op = op;
}

double PointArithRegion::GetX()
{
  if ( op == PTADD ) {
    return pt1->GetX() + pt2->GetX();
  }
  else if ( op == PTSUBTRACT ) {
    return pt1->GetX() - pt2->GetX();
  }
  else if ( op == PTMULTIPLY ) {
    return pt1->GetX() * pt2->GetX();
  }
  else if ( op == PTDIVIDE ) {
    return pt1->GetX() / pt2->GetX();
  }
  return UnknownDoubleValue;
}

double PointArithRegion::GetY()
{
  if ( op == PTADD ) {
    return pt1->GetY() + pt2->GetY();
  }
  else if ( op == PTSUBTRACT ) {
    return pt1->GetY() - pt2->GetY();
  }
  else if ( op == PTMULTIPLY ) {
    return pt1->GetY() * pt2->GetY();
  }
  else if ( op == PTDIVIDE ) {
    return pt1->GetY() / pt2->GetY();
  }
  return UnknownDoubleValue;
}

void PointArithRegion::Output( ostream & os, bool dv )
{
  os << '(';
  pt1->Output( os );
  if ( op == PTADD ) {
    os << " + ";
  }
  else if ( op == PTSUBTRACT ) {
    os << " - ";
  }
  else if ( op == PTMULTIPLY ) {
    os << " * ";
  }
  else if ( op == PTDIVIDE ) {
    os << " / ";
  }
  pt2->Output( os );
  os << ')';
}

// RectRegion stuff
RectRegion::RectRegion()
{
  tl = br = NULL;
}

RectRegion::~RectRegion()
{
  delete tl;
  delete br;
}

RectRegion::RectRegion( PointRegion * tl, PointRegion * br )
{
  this->tl = tl;
  this->br = br;
}

bool RectRegion::PointInRegion( double x, double y )
{
  double x1 = tl->GetX(), y1 = tl->GetY();
  double x2 = br->GetX(), y2 = br->GetY();

  if ( x1 > x2 ) {
    double tmp = x1;
    x1 = x2;
    x2 = tmp;
  }

  if ( y1 > y2 ) {
    double tmp = y1;
    y1 = y2;
    y2 = tmp;
  }

  if ( x < x1 || y < y1 || x > x2 || y > y2 )
    return false;
  else
    return true;
}

void RectRegion::Output( ostream & os, bool dv )
{
  os << "(rec ";
  tl->Output( os );
  os << ' ';
  br->Output( os );
  os << ')';
}

double RectRegion::Area()
{
  double x1 = tl->GetX();
  double x2 = br->GetX();
  double y1 = tl->GetY();
  double y2 = br->GetY();

  return fabs( x1 - x2 ) * fabs( y1 - y2 );
}

void RectRegion::GetClosestPointInRegion( double x, double y, double *rx,
					  double *ry )
{
  // Basic algorithm (used for rect, tri, and quad)
  // If point is in region, return point, otherwise:
  // For each side of polygon:
  //    - Calculate where the perpindicular between (x,y) and side
  //      intersects
  //    - If the point is on the side, compare and set with minimum
  // If minimum is set, return that, otherwise
  // Return closest vertex to point

  if ( PointInRegion( x, y ) ) {
    *rx = x;
    *ry = y;
    return;
  }

  double mindist2 = 999999;
  VecPosition p1, p2, ip, cp( x, y );
  double pt1x = tl->GetX();
  double pt1y = tl->GetY();
  double pt2x = br->GetX();
  double pt2y = br->GetY();
  if ( pt1x > pt2x ) {
    double tmp = pt1x;
    pt1x = pt2x;
    pt2x = tmp;
  }

  if ( pt1y > pt2y ) {
    double tmp = pt1y;
    pt1y = pt2y;
    pt2y = tmp;
  }

  Line l( 1, 1, 1 );
  double chkx[4] = { pt1x, pt2x, pt2x, pt1x };
  double chky[4] = { pt1y, pt1y, pt2y, pt2y };

  for ( int i = 0; i < 4; i++ ) {
    p1.setVecPosition( chkx[i], chky[i] );
    p2.setVecPosition( chkx[( i + 1 ) % 4], chky[( i + 1 ) % 4] );
    l = Line::makeLineFromTwoPoints( p1, p2 );
    ip = l.getPointOnLineClosestTo( cp );
    if ( l.isInBetween( ip, p1, p2 ) ) {
      double dist2 =
	( ip.getX() - x ) * ( ip.getX() - x ) + ( ip.getY() -
						      y ) * ( ip.getY() -
							      y );
      if ( dist2 < mindist2 ) {
	*rx = ip.getX();
	*ry = ip.getY();
	mindist2 = dist2;
      }
    }

  }

  if ( mindist2 != 999999 ) {
    return;
  }

  for ( int i = 0; i < 4; i++ ) {
    p1.setVecPosition( chkx[i], chky[i] );
    if ( p1.getDistanceTo( cp ) < mindist2 ) {
      *rx = chkx[i];
      *ry = chky[i];
      mindist2 = p1.getDistanceTo( cp );
    }
  }
}

// TriRegion stuff
TriRegion::TriRegion()
{
  pt1 = pt2 = pt3 = NULL;
}

TriRegion::~TriRegion()
{
  delete pt1;
  delete pt2;
  delete pt3;
}

TriRegion::TriRegion( PointRegion * pt1, PointRegion * pt2,
		      PointRegion * pt3 )
{
  this->pt1 = pt1;
  this->pt2 = pt2;
  this->pt3 = pt3;
}

bool TriRegion::PointInRegion( double x0, double y0 )
{
  double b0, b1, b2, b3;
  double x1, x2, x3;
  double y1, y2, y3;

  x1 = pt1->GetX();
  x2 = pt2->GetX();
  x3 = pt3->GetX();

  y1 = pt1->GetY();
  y2 = pt2->GetY();
  y3 = pt3->GetY();

  b0 = ( x2 - x1 ) * ( y3 - y1 ) - ( x3 - x1 ) * ( y2 - y1 );
  b1 = ( ( x2 - x0 ) * ( y3 - y0 ) - ( x3 - x0 ) * ( y2 - y0 ) ) / b0;
  b2 = ( ( x3 - x0 ) * ( y1 - y0 ) - ( x1 - x0 ) * ( y3 - y0 ) ) / b0;
  b3 = ( ( x1 - x0 ) * ( y2 - y0 ) - ( x2 - x0 ) * ( y1 - y0 ) ) / b0;

  return ( b1 >= 0 && b2 >= 0 && b3 >= 0 );
}

void TriRegion::Output( ostream & os, bool dv )
{
  os << "(tri ";
  pt1->Output( os );
  os << ' ';
  pt2->Output( os );
  os << ' ';
  pt3->Output( os );
  os << ')';
}

double TriRegion::Area()
{
  double x1 = pt1->GetX();
  double x2 = pt2->GetX();
  double x3 = pt3->GetX();

  double y1 = pt1->GetY();
  double y2 = pt2->GetY();
  double y3 = pt3->GetY();

  double base = sqrt( ( x1 - x2 ) * ( x1 - x2 ) + ( y1 - y2 ) * ( y1 - y2 ) );
  double height = Line::makeLineFromTwoPoints( VecPosition( x1, y1 ),
					       VecPosition( x2,
							    y2 ) ).
    getDistanceWithPoint( VecPosition( x3, y3 ) );

  return .5 * base * height;
}

void TriRegion::GetClosestPointInRegion( double x, double y, double *rx,
					 double *ry )
{
  if ( PointInRegion( x, y ) ) {
    *rx = x;
    *ry = y;
    return;
  }

  double mindist2 = 999999;
  VecPosition p1, p2, ip, cp( x, y );
  double pt1x = pt1->GetX();
  double pt1y = pt1->GetY();
  double pt2x = pt2->GetX();
  double pt2y = pt2->GetY();
  double pt3x = pt3->GetX();
  double pt3y = pt3->GetY();
  Line l( 1, 1, 1 );
  double chkx[3] = { pt1x, pt2x, pt3x };
  double chky[3] = { pt1y, pt2y, pt3y };

  for ( int i = 0; i < 3; i++ ) {
    p1.setVecPosition( chkx[i], chky[i] );
    p2.setVecPosition( chkx[( i + 1 ) % 3], chky[( i + 1 ) % 3] );
    l = Line::makeLineFromTwoPoints( p1, p2 );
    ip = l.getPointOnLineClosestTo( cp );
    if ( l.isInBetween( ip, p1, p2 ) ) {
      double dist2 =
	( ip.getX() - x ) * ( ip.getX() - x ) + ( ip.getY() -
						      y ) * ( ip.getY() -
							      y );
      if ( dist2 < mindist2 ) {
	*rx = ip.getX();
	*ry = ip.getY();
	mindist2 = dist2;
      }
    }

  }

  if ( mindist2 != 999999 ) {
    return;
  }

  for ( int i = 0; i < 3; i++ ) {
    p1.setVecPosition( chkx[i], chky[i] );
    if ( p1.getDistanceTo( cp ) < mindist2 ) {
      *rx = chkx[i];
      *ry = chky[i];
      mindist2 = p1.getDistanceTo( cp );
    }
  }
}

// NullRegion stuff

void NullRegion::Output( ostream & os, bool dv )
{
  os << "(null)";
}

// Justin says:
//   I honestly don't understand the point of the null region even existing
//   but just in case someone uses it, I'm going to make this one return 
//   (999999, 999999) so that a (reg ... (null) ....) will still return 
//   a good point. Otherwise, it should be some sort of undefined value

void NullRegion::GetClosestPointInRegion( double x, double y, double *rx,
					  double *ry )
{
  *rx = 999999;
  *ry = 999999;
}

// ArcRegion stuff
ArcRegion::ArcRegion()
{
  c = NULL;
}

ArcRegion::~ArcRegion()
{
  delete c;
}

ArcRegion::ArcRegion( PointRegion * c, double smrad, double lgrad,
		      double agbeg, double agend )
{
  this->c = c;
  this->smrad = smrad;
  this->lgrad = lgrad;
  this->begang = agbeg;
  this->spanang = agend;

}

bool ArcRegion::PointInRegion( double x, double y )
{
  double cx = c->GetX();
  double cy = c->GetY();
  double ang = atan2( ( y - cy ), ( x - cx ) ) * 180 / 3.1415926f;
  double dist2 = ( x - cx ) * ( x - cx ) + ( y - cy ) * ( y - cy );
  //double spang = spanang;

  if ( ( dist2 < smrad * smrad ) || ( dist2 > lgrad * lgrad ) )
    return false;

  double ang_diff = VecPosition::normalizeAngle(ang - begang);
  if (spanang >= 0)
    {
      if (ang_diff < 0)
	ang_diff += 360;
      return ang_diff <= spanang;
    }
  else
    {
      if (ang_diff > 0)
	ang_diff -= 360;
      return ang_diff >= spanang;
    }
}

double ArcRegion::Area()
{
  double totang = spanang - begang;
  if ( totang < 0 )
    totang += 360;

  totang /= 360;

  return ( ( 3.1415926 * lgrad * lgrad ) -
	   ( 3.1415926 * smrad * smrad ) ) * totang;
}

void ArcRegion::Output( ostream & os, bool dv )
{
  os << "(arc ";
  c->Output( os );
  os << ' ' << smrad << ' ' << lgrad << ' ' << begang << ' ' << spanang <<
    ')';
}

void ArcRegion::GetClosestPointInRegion( double x, double y, double *rx,
					 double *ry )
{

  // If point is in region, return point
  if ( PointInRegion( x, y ) ) {
    *rx = x;
    *ry = y;
    return;
  }
  // Otherwise there is a total of 8 points to check
  // The intersection of the line (x,y)-(cx,cy) with
  // circles of smrad and bigrad (throw them out if
  // they aren't within the arc)
  // and the endpoints of each arc

  double cx = c->GetX(), cy = c->GetY();

  double chx[8], chy[8];

  CircleIntersect( cx, cy, smrad, x, y, &chx[0], &chy[0], &chx[1], &chy[1] );
  CircleIntersect( cx, cy, lgrad, x, y, &chx[2], &chy[2], &chx[3], &chy[3] );

  double bc = cos( begang * 3.1415926 / 180 );
  double bs = sin( begang * 3.1415926 / 180 );
  double sc = cos( spanang * 3.1415926 / 180 );
  double ss = sin( spanang * 3.1415926 / 180 );

  chx[4] = cx + smrad * bc;
  chy[4] = cy + smrad * bs;

  chx[5] = cx + smrad * sc;
  chy[5] = cy + smrad * ss;

  chx[6] = cx + lgrad * bc;
  chy[6] = cy + lgrad * bs;

  chx[7] = cx + lgrad * sc;
  chy[7] = cy + lgrad * ss;

  double mindist2 = 9999999;
  double spang = spanang;
  double ang, dist2;

  for ( int i = 0; i < 4; i++ ) {
    ang = atan2( ( chy[i] - cy ), ( chx[i] - cx ) ) * 180 / 3.1415926f;

    if ( ang < 0 )
      ang += 360;
    else if ( ang > 359 )
      ang -= 360;

    if ( begang > spanang ) {
      spang = spanang + 360;
      ang += 360;
    }

    if ( ang >= begang && ang <= spanang ) {
      dist2 =
	( chy[i] - y ) * ( chy[i] - y ) + ( chx[i] - x ) * ( chx[i] - x );
      if ( dist2 < mindist2 ) {
	mindist2 = dist2;
	*rx = chx[i];
	*ry = chy[i];
      }
    }
    else {
    }
  }

  for ( int i = 4; i < 8; i++ ) {
    dist2 = ( chy[i] - y ) * ( chy[i] - y ) + ( chx[i] - x ) * ( chx[i] - x );
    if ( dist2 < mindist2 ) {
      mindist2 = dist2;
      *rx = chx[i];
      *ry = chy[i];
    }
  }

}

// UnionRegion stuff
UnionRegion::UnionRegion( regionList * r )
{
  reglist = r;
}

UnionRegion::~UnionRegion()
{
  regionList *cur = reglist, *tmp;
  while ( cur ) {
    tmp = cur;
    cur = cur->next;
    delete tmp->reg;
    delete tmp;
  }
}

bool UnionRegion::PointInRegion( double x, double y )
{
  regionList *cur = reglist;
  while ( cur ) {
    if ( cur->reg->PointInRegion( x, y ) )
      return true;
    cur = cur->next;
  }
  return false;
}

void UnionRegion::Output( ostream & os, bool dv )
{
  regionList *cur = reglist;
  os << "(reg ";
  while ( cur ) {
    cur->reg->Output( os );
    if ( cur->next )
      os << ' ';
    cur = cur->next;
  }
  os << ')';
}

/// Note, this naively assumes that the regions are all non-intersecting
double UnionRegion::Area()
{
  regionList *cur = reglist;
  double totarea = 0;
  while ( cur ) {
    totarea += cur->reg->Area();
    cur = cur->next;
  }
  return totarea;
}

void UnionRegion::GetClosestPointInRegion( double x, double y, double *rx,
					   double *ry )
{
  double bestdist2 = 1000000, testx, testy, dist2;
  regionList *cur = reglist;
  while ( cur ) {
    cur->reg->GetClosestPointInRegion( x, y, &testx, &testy );
    dist2 = ( x - testx ) * ( x - testx ) + ( y - testy ) * ( y - testy );
    if ( dist2 < bestdist2 ) {
      bestdist2 = dist2;
      *rx = testx;
      *ry = testy;
    }
    cur = cur->next;
  }

}

/// VarPtRegion stuff

void VarPtRegion::Output( ostream & os, bool dv )
{
  if ( dv )
    v->Output( os );
  else
    rulebase->ResolvePtRegion( v )->Output( os, dv );
}

double VarPtRegion::GetX()
{
  return rulebase->ResolvePtRegion( v )->GetX();
}

double VarPtRegion::GetY()
{
  return rulebase->ResolvePtRegion( v )->GetY();
}

void VarRegion::Output( ostream & os, bool dv )
{
  if ( dv )
    v->Output( os );
  else
    rulebase->ResolveRegion( v )->Output( os, dv );
}

bool VarRegion::PointInRegion( double x, double y )
{
  return rulebase->ResolveRegion( v )->PointInRegion( x, y );
}

double VarRegion::Area()
{
  return rulebase->ResolveRegion( v )->Area();
}

void VarRegion::GetClosestPointInRegion( double x, double y, double *rx,
					 double *ry )
{
  rulebase->ResolveRegion( v )->GetClosestPointInRegion( x, y, rx, ry );
}

void VarRegion::SetAssocRule( Rule * r )
{
  rulebase->ResolveRegion( v )->SetAssocRule( r );
}

void VarPtRegion::SetAssocRule( Rule * r )
{
  rulebase->ResolvePtRegion( v )->SetAssocRule( r );
}

// UnumSetPoint Stuff

double UnumSetPoint::GetX()
{
  if ( !r )
    return -999999;

  UnumSet *s = r->GetInstantiatedVariable( v );
  if ( !s ) {
    return -999999;
  }

  int p = -1;
  for ( int i = 1; i <= 11; i++ ) {
    if ( s->IsSet( i ) ) {
      if ( p != -1 )
	cerr <<
	  "Warning: Attempting to use a variable that has been instantiated to more than one Unum for a UnumSetPoint."
	  << endl;
      p = i;
    }
  }
  return CLGetPlayerX( t, p );
}

double UnumSetPoint::GetY()
{
  if ( !r )
    return -999999;
  UnumSet *s = r->GetInstantiatedVariable( v );
  if ( !s ) {
    return -999999;
  }
  int p = -1;

  for ( int i = 1; i <= 11; i++ ) {
    if ( s->IsSet( i ) ) {
      if ( p != -1 )
	cerr <<
	  "Warning: Attempting to use a variable that has been instantiated to more than one Unum for a UnumSetPoint."
	  << endl;
      p = i;
    }
  }
  return CLGetPlayerY( t, p );

}

void UnumSetPoint::Output( ostream & os, bool dv )
{

  os << "(pt ";
  if ( t == our )
    os << "our ";
  else
    os << "opp ";
  if ( !r || dv ) {
    v->Output( os );
    os << ")";
    return;
  }

  UnumSet *s = r->GetInstantiatedVariable( v );
  if ( !s )
    v->Output( os );
  else
    s->Output( os, dv );
  os << ")";
}

void CircleIntersect( double cx, double cy, double rad, double tx, double ty,
		      double *rx1, double *ry1, double *rx2, double *ry2 )
{
  double tang = atan2( ty - cy, tx - cx );
  double ac = cos( tang );
  double as = sin( tang );
  *rx1 = cx + rad * ac;
  *ry1 = cy + rad * as;
  *rx2 = cx - rad * ac;
  *ry2 = cy - rad * as;
}
